#include <stdio.h>

#include "mem.h"
#include "pagetable.h"

void page_info(uint32_t logic) {
    page_entry_t *pe;

    pe = pagetable_get(logic);

    if (pe == NULL) {
        printf("0x%x: no mapping.\n", logic);
        return;
    }

    printf("0x%x: physical=0x%x writeable=%d readable=%d.\n",
            logic,
            pe->physical,
            (pe->protection & MEM_WRITE) != 0,
            (pe->protection & MEM_READ) != 0
    );
}

int main() {
    page_entry_t pe;
	uint32_t physical;

    pagetable_init();	// Intialisr sidetabel

	/*
	Vi indstter elementer i sidetabellen og tester, om de er blevet korrekt indsat 
	(dette er den oprindelige udleverede test, umodificeret).
	*/
	
	printf("\n************ BEGYND UDLEVERET TEST ************\n\n");

    pe.physical = 0x1000;
    pe.protection = MEM_READ;
    pagetable_set(0x56000, &pe);

    pe.physical = 0x23000;
    pe.protection = MEM_READ | MEM_WRITE;
    pagetable_set(0x80000, &pe);

    page_info(0x56000);
    page_info(0x560f0);
    page_info(0x80fff);

    page_info(0x81fff);


	/*
	Vi indstter nu et element mere i sidetabellen og tester med 
	mem_read(), om siderne kan tilgs korrekt mht. tilladelser.
	De to frste skulle gerne g godt, den tredje skulle gerne give en protection 
	fault, og den sidste skulle gerne give en illegal address fault.
	*/
	
	printf("\n************ BEGYND EGEN TEST, mem_read() ************\n\n");

	pe.physical = 0x10000;
    pe.protection = MEM_WRITE;
    pagetable_set(0x17000, &pe);

	// Forsg at lse side med bde MEM_READ og MEM_WRITE tilladelser:
	physical = mem_read(0x80000);
	if(physical != 0)
		printf("mem_read() lst 0x80000, fysisk adresse retur: 0x%x\n", physical);
		
	// Forsg at lse side med kun MEM_READ tilladelser:
	physical = mem_read(0x56000);
	if(physical != 0)
		printf("mem_read() lst 0x56000, fysisk adresse retur: 0x%x\n", physical);
		
	// Forsg at lse side med kun MEM_WRITE tilladelser:
	physical = mem_read(0x17000);
	if(physical != 0)
		printf("mem_read() lst 0x17000, fysisk adresse retur: 0x%x\n", physical);
	
	// Forsg at lse side, der ikke findes i TLB eller sidetabel:
	physical = mem_read(0x100000);
	if(physical != 0)
		printf("mem_read() lst 0x100000, fysisk adresse retur: 0x%x\n", physical);


	/*
	Vi tester nu mem_write() p de eksisterende indgange i sidetabellen.
	De to frste skulle gerne g godt, den tredje skulle gerne give en protection 
	fault, og den sidste skulle gerne give en illegal address fault.
	*/
	
	printf("\n************ BEGYND EGEN TEST, mem_write() ************\n\n");

	// Forsg at skrive side med bde MEM_READ og MEM_WRITE tilladelser:
	physical = mem_write(0x80000);
	if(physical != 0)
		printf("mem_write() skrevet 0x80000, fysisk adresse retur: 0x%x\n", physical);
		
	// Forsg at lse side med kun MEM_WRITE tilladelser:
	physical = mem_write(0x17000);
	if(physical != 0)
		printf("mem_write() skrevet 0x17000, fysisk adresse retur: 0x%x\n", physical);
		
	// Forsg at lse side med kun MEM_WRITE tilladelser:
	physical = mem_write(0x56000);
	if(physical != 0)
		printf("mem_write() skrevet 0x56000, fysisk adresse retur: 0x%x\n", physical);
	
	// Forsg at lse side, der ikke findes i TLB eller sidetabel:
	physical = mem_write(0x100000);
	if(physical != 0)
		printf("mem_write() skrevet 0x100000, fysisk adresse retur: 0x%x\n", physical);

	printf("\n");

    return 0;
}
