#include <inttypes.h>
#include <stdio.h>

#include "mem.h"
#include "tlb.h"
#include "pagetable.h"

/*
Svarer til at en proces lser den logiske adresse 'logical'.
Den tilsvarende fysiske adresse returneres, eller '0' ved fejl.
*/
uint32_t mem_read(uint32_t logical) {
	uint32_t physical;
	uint16_t protection;
	
	protection = tlb_lookup(logical, &physical);
	
	if(protection == 0) {	// Adressen findes ikke i hverken TLB eller sidetabel
		printf("FEJL: Illegal address fault!\n");
		return 0;
	}
	
	if(protection & 1) {	// Der m lses fra filen
		return physical;
	}
	else {					// Der m ikke lses fra filen
		printf("FEJL: Protection fault!\n");
		return 0;
	}
}

/*
Svarer til at en proces skriver den logiske adresse 'logical'.
Den tilsvarende fysiske adresse returneres, eller '0' ved fejl.
*/
uint32_t mem_write(uint32_t logical) {
	uint32_t physical;
	uint16_t protection;
	
	protection = tlb_lookup(logical, &physical);
	
	if(protection == 0) {		// Adressen findes ikke i hverken TLB eller sidetabel
		printf("FEJL: Illegal address fault!\n");
		return 0;
	}
	
	if((protection >> 1) & 1) {	// Der m skrives til filen
		return physical;
	}
	else {						// Der m ikke skrives til filen
		printf("FEJL: Protection fault!\n");
		return 0;
	}
}

/*
Hvis TLB'en ikke kan finde den logiske adresse 'logical'
i 'tlb_lookup' kaldes denne funktion. Hvis den returnerer
'0' stopper TLB'en med opslaget, ellers prver den igen.
*/
int tlb_fault(uint32_t logical) {
	page_entry_t *pe;
	
	pe = pagetable_get(logical);	// Forsg at hente siden fra sidetabellen
	
	if(pe == NULL) {				// Adressen eksisterer ikke; stop TLB sgning
		return 0;
	} 
	else {							// Indst siden i TLB'en og gennemsg denne igen
		tlb_insert(logical, pe->physical, pe->protection);
		return 1;
	}
}
