#include <stdio.h>

#include "tlb.h"
#include "mem.h"

int tlb_fault(uint32_t logical) {
    return 0;
}

void page_info(uint32_t logic) {
    uint32_t phys;
    uint16_t prot;

    prot = tlb_lookup(logic, &phys);
    if (prot) {
        printf("0x%x: physical=0x%x writeable=%d readable=%d.\n",
            logic,
            phys,
            (prot & MEM_WRITE) != 0,
            (prot & MEM_READ) != 0
        );
    } else {
        printf("0x%x: no mapping.\n", logic);
    }
}


int main() {

    tlb_init();	// Initialisr TLB'en

    tlb_insert(0x16000, 0x1000, MEM_READ | MEM_WRITE);
	tlb_insert(0x26000, 0x1010, MEM_READ);
	tlb_insert(0x36000, 0x1020, MEM_READ | MEM_WRITE);
	page_info(0x16000);	// Tjek, om tlb_insert() fungerer korrekt
	tlb_insert(0x46000, 0x1030, MEM_READ);
	tlb_insert(0x56000, 0x1040, MEM_READ | MEM_WRITE);
	tlb_insert(0x66000, 0x1050, MEM_READ);
	page_info(0x26000); // Tjek, om tlb_insert() fungerer korrekt
	tlb_insert(0x76000, 0x1060, MEM_READ | MEM_WRITE);
	tlb_insert(0x86000, 0x1070, MEM_READ);
	page_info(0x36000); // Tjek, om tlb_insert() fungerer korrekt
	
	/*
	TLB'en er nu fuld, og alle entries har sat deres TLB_REFERENCED.
	Ved nste insert br second chance lbe alle igennem, 
	rydde TLB_REFERENCED og erstatte plads 0 med det nye element.
	*/
	
	tlb_insert(0x96000, 0x1080, MEM_READ | MEM_WRITE);
	page_info(0x26000);
	page_info(0x36000);
	page_info(0x46000);
	page_info(0x66000);
	
	/*
	De 4 page_info()-kald ovenfor stter TLB_REFERENCED til true 
	p plads 1, 2, 3 og 5. Ved nste indsttelse skulle second-chance 
	derfor gerne ramme plads 4.
	*/
	
	tlb_insert(0x106000, 0x1090, MEM_READ);
	page_info(0x106000); // Tjek, om tlb_insert() fungerer korrekt
	
    page_info(0xFFF000); // Forsger at tilg logisk adresse, der ikke findes i TLB'en

    return 0;
}
